@@ -20,16 +20,16 @@
 #          Christopher Williams (christopherw@verizon.net)
 # Copyright (C) 2007 Quinn Storm
 
-import pygtk
-import gtk
-import gobject
-import os
+from gi.repository import GLib
+from gi.repository import Gtk
+from gi.repository import Gdk
 
-from ccm.Constants import *
-from ccm.Conflicts import *
-from ccm.Widgets import *
-from ccm.Utils import *
-from ccm.Pages import *
+from ccm.Constants import TableDef, TableX, KeyModifier, Edges, ImageStock, FilterName, FilterLongDesc, FilterValue, FilterAll, DataDir
+from ccm.Conflicts import KeyConflict, ButtonConflict, EdgeConflict
+from ccm.Widgets import CellRendererColor, ModifierSelector, SingleEdgeSelector, KeyGrabber, MatchButton, FileButton, ErrorDialog
+from ccm.Utils import Image, ActionImage, SizedButton, GlobalUpdater, PureVirtualError, SettingKeyFunc, EnumSettingKeyFunc, HasOnlyType, GetSettings, GetAcceleratorName
+
+from cgi import escape as protect_pango_markup
 
 import locale
 import gettext
@@ -52,21 +52,21 @@ class Setting(object):
             self.CurrentRow = None
 
         self.Blocked = 0
-        self.EBox = gtk.EventBox()
-        self.Box = gtk.HBox()
+        self.EBox = Gtk.EventBox()
+        self.Box = Gtk.HBox()
         self.EBox.set_visible_window(False)
         if Setting:
             self.EBox.set_sensitive(not Setting.ReadOnly)
         self.Box.set_spacing(5)
         self.EBox.add(self.Box)
-        self.Reset = gtk.Button()
+        self.Reset = Gtk.Button()
         if not Settings:
             self.MakeLabel()
             markup = "%s\n<small><i>%s</i></small>" % (self.Setting.LongDesc, self.Setting.Name)
             self.EBox.set_tooltip_markup(markup)
             self.Reset.set_tooltip_text(_("Reset setting to the default value"))
-        self.Reset.set_image (Image (name = gtk.STOCK_CLEAR, type = ImageStock,
-                                     size = gtk.ICON_SIZE_BUTTON))
+        self.Reset.set_image (Image (name = Gtk.STOCK_CLEAR, type = ImageStock,
+                                     size = Gtk.IconSize.BUTTON))
         self.Reset.connect('clicked', self.DoReset)
         self._Init()
 
@@ -84,7 +84,7 @@ class Setting(object):
         self.RemoveUpdater()
 
     def GetColumn(self, num):
-        return (str, gtk.TreeViewColumn(self.Setting.ShortDesc, gtk.CellRendererText(), text=num))
+        return (str, Gtk.TreeViewColumn(self.Setting.ShortDesc, Gtk.CellRendererText(), text=num))
 
     def PureVirtual (self, func):
         message = "Missing %(function)s function for %(name)s setting (%(class)s)"
@@ -109,7 +109,7 @@ class Setting(object):
         if not self.Setting:
             return
 
-        label = gtk.Label()
+        label = Gtk.Label()
         desc = protect_pango_markup (self.Setting.ShortDesc)
         style = "%s"
         if self.Setting.Integrated:
@@ -117,7 +117,7 @@ class Setting(object):
         label.set_markup(style % desc)
         label.props.xalign = 0
         label.set_size_request(160, -1)
-        label.props.wrap_mode = pango.WRAP_WORD
+        label.props.wrap_mode = Gtk.WrapMode.WORD
         label.set_line_wrap(True)
         self.Label = label
 
@@ -206,17 +206,17 @@ class Setting(object):
 class StockSetting(Setting):
 
     def _Init(self):
-        self.Box.pack_start(self.Label, False, False)
-        self.Box.pack_end(self.Reset, False, False)
+        self.Box.pack_start(self.Label, False, False, 0)
+        self.Box.pack_end(self.Reset, False, False, 0)
 
 class StringSetting(StockSetting):
     def _Init(self):
         StockSetting._Init(self)
-        self.Entry = gtk.Entry()
+        self.Entry = Gtk.Entry()
         self.Entry.connect('activate', self.Changed)
         self.Entry.connect('focus-out-event', self.Changed)
         self.Widget = self.Entry
-        self.Box.pack_start(self.Widget, True, True)
+        self.Box.pack_start(self.Widget, True, True, 0)
 
     def _Read(self):
         self.Entry.set_text(self.Get())
@@ -228,7 +228,7 @@ class MatchSetting(StringSetting):
     def _Init(self):
         StringSetting._Init(self)
         self.MatchButton = MatchButton(self.Entry)
-        self.Box.pack_start(self.MatchButton, False, False)
+        self.Box.pack_start(self.MatchButton, False, False, 0)
 
 class FileStringSetting(StringSetting):
 
@@ -241,7 +241,7 @@ class FileStringSetting(StringSetting):
         StringSetting._Init(self)
         self.FileButton = FileButton(self.Setting.Plugin.Context, self.Entry,
             self.isDirectory, self.isImage)
-        self.Box.pack_start(self.FileButton, False, False)
+        self.Box.pack_start(self.FileButton, False, False, 0)
 
 class EnumSetting(StockSetting):
 
@@ -249,7 +249,7 @@ class EnumSetting(StockSetting):
 
     def _Init(self):
         StockSetting._Init(self)
-        self.Combo = gtk.combo_box_new_text()
+        self.Combo = Gtk.ComboBoxText()
         if self.List:
             self.Info = self.Setting.Info[1][2]
         else:
@@ -260,7 +260,7 @@ class EnumSetting(StockSetting):
         self.Combo.connect('changed', self.Changed)
 
         self.Widget = self.Combo
-        self.Box.pack_start(self.Combo, True, True)
+        self.Box.pack_start(self.Combo, True, True, 0)
 
     def _CellEdited(self, cell, path, new_text):
         self.CurrentRow = int(path[0])
@@ -271,9 +271,9 @@ class EnumSetting(StockSetting):
 
     def GetColumn(self, num):
         self.Num = num
-        cell = gtk.CellRendererCombo()
-        column = gtk.TreeViewColumn(self.Setting.ShortDesc, cell, text=num)
-        model = gtk.ListStore(str)
+        cell = Gtk.CellRendererCombo()
+        column = Gtk.TreeViewColumn(self.Setting.ShortDesc, cell, text=num)
+        model = Gtk.ListStore(str)
         for property, value in [("model", model), ("text_column", 0),
                                 ("editable", False), ("has_entry", False)]:
             cell.set_property (property, value)
@@ -306,7 +306,7 @@ class RestrictedStringSetting(StockSetting):
 
     def _Init(self):
         StockSetting._Init(self)
-        self.Combo = gtk.combo_box_new_text()
+        self.Combo = Gtk.ComboBoxText()
         if self.List:
             info = self.Setting.Info[1]
         else:
@@ -324,7 +324,7 @@ class RestrictedStringSetting(StockSetting):
         self.Combo.connect('changed', self.Changed)
 
         self.Widget = self.Combo
-        self.Box.pack_start(self.Combo, True, True)
+        self.Box.pack_start(self.Combo, True, True, 0)
 
         self.OriginalValue = None
         self.NAItemShift = 0
@@ -338,9 +338,9 @@ class RestrictedStringSetting(StockSetting):
 
     def GetColumn(self, num):
         self.Num = num
-        cell = gtk.CellRendererCombo()
-        column = gtk.TreeViewColumn(self.Setting.ShortDesc, cell, text=num)
-        model = gtk.ListStore(str)
+        cell = Gtk.CellRendererCombo()
+        column = Gtk.TreeViewColumn(self.Setting.ShortDesc, cell, text=num)
+        model = Gtk.ListStore(str)
         for property, value in [("model", model), ("text_column", 0),
                                 ("editable", False), ("has_entry", False)]:
             cell.set_property (property, value)
@@ -399,10 +399,10 @@ class BoolSetting (StockSetting):
     def _Init (self):
         StockSetting._Init(self)
         self.Label.set_size_request(-1, -1)
-        self.CheckButton = gtk.CheckButton ()
-        align = gtk.Alignment(yalign=0.5)
+        self.CheckButton = Gtk.CheckButton ()
+        align = Gtk.Alignment(yalign=0.5)
         align.add(self.CheckButton)
-        self.Box.pack_end(align, False, False)
+        self.Box.pack_end(align, False, False, 0)
         self.CheckButton.connect ('toggled', self.Changed)
 
     def _Read (self):
@@ -419,10 +419,10 @@ class BoolSetting (StockSetting):
 
     def GetColumn (self, num):
         self.Num = num
-        cell = gtk.CellRendererToggle()
+        cell = Gtk.CellRendererToggle()
         cell.set_property("activatable", True)
         cell.connect('toggled', self.CellToggled)
-        return (bool, gtk.TreeViewColumn(self.Setting.ShortDesc, cell, active=num))
+        return (bool, Gtk.TreeViewColumn(self.Setting.ShortDesc, cell, active=num))
 
 class NumberSetting(StockSetting):
 
@@ -439,19 +439,13 @@ class NumberSetting(StockSetting):
             self.Inc = info[2]
         inc = self.Inc
         self.NoneValue = info[0]
-        self.Adj = gtk.Adjustment(0, info[0], info[1], inc, inc*10)
-        self.Spin = gtk.SpinButton(self.Adj)
+        self.Adj = Gtk.Adjustment(0, info[0], info[1], inc, inc*10)
+        self.Spin = Gtk.SpinButton(adjustment=self.Adj)
         self.Spin.set_value(self.Get())
-
-        self.Scale = gtk.HScale(self.Adj)
-
-        self.Scale.set_update_policy(gtk.UPDATE_DISCONTINUOUS)
-        self.Scale.connect("value-changed", self.Changed)
         self.Spin.connect("value-changed", self.Changed)
-        self.Widget = self.Scale
+        self.Widget = self.Spin
 
-        self.Box.pack_start(self.Scale, True, True)
-        self.Box.pack_start(self.Spin, False, False)
+        self.Box.pack_end(self.Spin, False, False, 0)
 
     def _Read(self):
         self.Adj.set_value(self.Get())
@@ -465,7 +459,6 @@ class IntSetting(NumberSetting):
         self.Inc = 1
         NumberSetting._Init(self)
         self.Spin.set_digits(0)
-        self.Scale.set_digits(0)
 
 class FloatSetting(NumberSetting):
 
@@ -475,7 +468,6 @@ class FloatSetting(NumberSetting):
         self.Inc = None
         NumberSetting._Init(self)
         self.Spin.set_digits(4)
-        self.Scale.set_digits(4)
 
 
 class ColorSetting(StockSetting):
@@ -484,25 +476,25 @@ class ColorSetting(StockSetting):
 
     def _Init(self):
         StockSetting._Init(self)
-        self.Button = gtk.ColorButton()
+        self.Button = Gtk.ColorButton()
         self.Button.set_size_request (100, -1)
         self.Button.set_use_alpha(True)
         self.Button.connect('color-set', self.Changed)
 
-        self.Widget = gtk.Alignment (1, 0.5)
+        self.Widget = Gtk.Alignment (xalign=1, yalign=0.5)
         self.Widget.add (self.Button)
-        self.Box.pack_start(self.Widget, True, True)
+        self.Box.pack_start(self.Widget, True, True, 0)
 
     def GetForRenderer(self):
         return ["#%.4x%.4x%.4x%.4x" %tuple(seq) for seq in self.Setting.Value]
 
     def GetColumn(self, num):
-        return (str, gtk.TreeViewColumn(self.Setting.ShortDesc, CellRendererColor(), text=num))
+        return (str, Gtk.TreeViewColumn(self.Setting.ShortDesc, CellRendererColor(), text=num))
 
     def _Read(self):
-        col = gtk.gdk.Color()
         value = self.Get()
-        col.red, col.green, col.blue = value[:3]
+        red, green, blue = value[:3]
+        col = Gdk.Color(red, green, blue)
         self.Button.set_color(col)
         self.Button.set_alpha(value[3])
 
@@ -513,7 +505,7 @@ class ColorSetting(StockSetting):
 
 class BaseListSetting(Setting):
     def _Init(self):
-        self.Widget = gtk.VBox()
+        self.Widget = Gtk.VBox()
         self.EditDialog = None        
         self.EditDialogOpen = False
         self.PageToBeRefreshed = None
@@ -524,8 +516,8 @@ class BaseListSetting(Setting):
 
         types, cols = self.ListInfo()
         self.Types = types
-        self.Store = gtk.ListStore(*types)
-        self.View = gtk.TreeView(self.Store)
+        self.Store = Gtk.ListStore(*types)
+        self.View = Gtk.TreeView(self.Store)
         self.View.set_headers_visible(True)
 
         for widget in self.Widgets:
@@ -539,29 +531,29 @@ class BaseListSetting(Setting):
         self.View.connect('button-press-event', self.ButtonPressEvent)
         self.View.connect('key-press-event', self.KeyPressEvent)
         self.Select = self.View.get_selection()
-        self.Select.set_mode(gtk.SELECTION_SINGLE)
+        self.Select.set_mode(Gtk.SelectionMode.SINGLE)
         self.Select.connect('changed', self.SelectionChanged)
         self.Widget.set_spacing(5)
-        self.Scroll = gtk.ScrolledWindow()
-        self.Scroll.props.hscrollbar_policy = gtk.POLICY_AUTOMATIC
-        self.Scroll.props.vscrollbar_policy = gtk.POLICY_NEVER
+        self.Scroll = Gtk.ScrolledWindow()
+        self.Scroll.props.hscrollbar_policy = Gtk.PolicyType.AUTOMATIC
+        self.Scroll.props.vscrollbar_policy = Gtk.PolicyType.NEVER
         self.Scroll.add(self.View)
-        self.Widget.pack_start(self.Scroll, True, True)
-        self.Widget.set_child_packing(self.Scroll, True, True, 0, gtk.PACK_START)
-        buttonBox = gtk.HBox(False)
+        self.Widget.pack_start(self.Scroll, True, True, 0)
+        self.Widget.set_child_packing(self.Scroll, True, True, 0, Gtk.PackType.START)
+        buttonBox = Gtk.HBox(False)
         buttonBox.set_spacing(5)
         buttonBox.set_border_width(5)
-        self.Widget.pack_start(buttonBox, False, False)
-        buttonTypes = ((gtk.STOCK_NEW, self.Add, None, True),
-                 (gtk.STOCK_DELETE, self.Delete, None, False), 
-                 (gtk.STOCK_EDIT, self.Edit, None, False),
-                 (gtk.STOCK_GO_UP, self.Move, 'up', False), 
-                 (gtk.STOCK_GO_DOWN, self.Move, 'down', False),)
+        self.Widget.pack_start(buttonBox, False, False, 0)
+        buttonTypes = ((Gtk.STOCK_NEW, self.Add, None, True),
+                 (Gtk.STOCK_DELETE, self.Delete, None, False), 
+                 (Gtk.STOCK_EDIT, self.Edit, None, False),
+                 (Gtk.STOCK_GO_UP, self.Move, 'up', False), 
+                 (Gtk.STOCK_GO_DOWN, self.Move, 'down', False),)
         self.Buttons = {}
         for stock, callback, data, sensitive in buttonTypes:
-            b = gtk.Button(stock)
+            b = Gtk.Button(stock)
             b.set_use_stock(True)
-            buttonBox.pack_start(b, False, False)
+            buttonBox.pack_start(b, False, False, 0)
             if data is not None:
                 b.connect('clicked', callback, data)
             else:
@@ -569,23 +561,23 @@ class BaseListSetting(Setting):
             b.set_sensitive(sensitive)
             self.Buttons[stock] = b
 
-        self.Popup = gtk.Menu()
+        self.Popup = Gtk.Menu()
         self.PopupItems = {}
-        edit = gtk.ImageMenuItem(stock_id=gtk.STOCK_EDIT)
+        edit = Gtk.ImageMenuItem.new_from_stock(stock_id=Gtk.STOCK_EDIT)
         edit.connect('activate', self.Edit)
         edit.set_sensitive(False)
         self.Popup.append(edit)
-        self.PopupItems[gtk.STOCK_EDIT] = edit
-        delete = gtk.ImageMenuItem(stock_id=gtk.STOCK_DELETE)
+        self.PopupItems[Gtk.STOCK_EDIT] = edit
+        delete = Gtk.ImageMenuItem.new_from_stock(stock_id=Gtk.STOCK_DELETE)
         delete.connect('activate', self.Delete)
         delete.set_sensitive(False)
         self.Popup.append(delete)
-        self.PopupItems[gtk.STOCK_DELETE] = delete
+        self.PopupItems[Gtk.STOCK_DELETE] = delete
         self.Popup.show_all()
 
-        buttonBox.pack_end(self.Reset, False, False)
+        buttonBox.pack_end(self.Reset, False, False, 0)
 
-        self.Box.pack_start(self.Widget)
+        self.Box.pack_start(self.Widget, True, True, 0)
 
     def AddUpdater(self):
         pass
@@ -635,17 +627,17 @@ class BaseListSetting(Setting):
             self._Delete(row)
 
     def _MakeEditDialog(self):
-        dlg = gtk.Dialog(_("Edit"))
-        vbox = gtk.VBox(spacing=TableX)
+        dlg = Gtk.Dialog(_("Edit"))
+        vbox = Gtk.VBox(spacing=TableX)
         vbox.props.border_width = 6
-        dlg.vbox.pack_start(vbox, True, True)
+        dlg.vbox.pack_start(vbox, True, True, 0)
         dlg.set_default_size(500, -1)
-        dlg.add_button(gtk.STOCK_CLOSE, gtk.RESPONSE_CLOSE)
-        dlg.set_default_response(gtk.RESPONSE_CLOSE)
+        dlg.add_button(Gtk.STOCK_CLOSE, Gtk.ResponseType.CLOSE)
+        dlg.set_default_response(Gtk.ResponseType.CLOSE)
 
-        group = gtk.SizeGroup(gtk.SIZE_GROUP_HORIZONTAL)
+        group = Gtk.SizeGroup(Gtk.SizeGroupMode.HORIZONTAL)
         for widget in self.Widgets:
-            vbox.pack_start(widget.EBox, False, False)
+            vbox.pack_start(widget.EBox, False, False, 0)
             group.add_widget(widget.Label)
         return dlg
 
@@ -670,8 +662,8 @@ class BaseListSetting(Setting):
 
         self.EditDialogOpen = True
         self.EditDialog.show_all()
-        response = self.EditDialog.run()
-        self.EditDialog.hide_all()
+        self.EditDialog.run()
+        self.EditDialog.hide()
         self.EditDialogOpen = False
 
         if self.PageToBeRefreshed:
@@ -707,19 +699,19 @@ class BaseListSetting(Setting):
     def SelectionChanged(self, selection):
 
         model, it = selection.get_selected()
-        for widget in (self.Buttons[gtk.STOCK_EDIT], self.Buttons[gtk.STOCK_DELETE],
-                       self.PopupItems[gtk.STOCK_EDIT], self.PopupItems[gtk.STOCK_DELETE]):
+        for widget in (self.Buttons[Gtk.STOCK_EDIT], self.Buttons[Gtk.STOCK_DELETE],
+                       self.PopupItems[Gtk.STOCK_EDIT], self.PopupItems[Gtk.STOCK_DELETE]):
             widget.set_sensitive(it is not None)
             
         if it is not None:
             path = model.get_path(it)
             if path is not None:
                 row = path[0]
-                self.Buttons[gtk.STOCK_GO_UP].set_sensitive(row > 0)
-                self.Buttons[gtk.STOCK_GO_DOWN].set_sensitive(row < (len(model) - 1))
+                self.Buttons[Gtk.STOCK_GO_UP].set_sensitive(row > 0)
+                self.Buttons[Gtk.STOCK_GO_DOWN].set_sensitive(row < (len(model) - 1))
         else:
-            self.Buttons[gtk.STOCK_GO_UP].set_sensitive(False)
-            self.Buttons[gtk.STOCK_GO_DOWN].set_sensitive(False)
+            self.Buttons[Gtk.STOCK_GO_UP].set_sensitive(False)
+            self.Buttons[Gtk.STOCK_GO_DOWN].set_sensitive(False)
 
     def ButtonPressEvent(self, treeview, event):
         if event.button == 3:
@@ -732,7 +724,7 @@ class BaseListSetting(Setting):
             return True
 
     def KeyPressEvent(self, treeview, event):
-        if gtk.gdk.keyval_name(event.keyval) == "Delete":
+        if Gdk.keyval_name(event.keyval) == "Delete":
             model, it = treeview.get_selection().get_selected()
             if it is not None:
                 path = model.get_path(it)
@@ -791,15 +783,15 @@ class MultiListSetting(BaseListSetting):
 class EnumFlagsSetting(Setting):
 
     def _Init(self):
-        frame = gtk.Frame(self.Setting.ShortDesc)
-        table = gtk.Table()
+        frame = Gtk.Frame(label=self.Setting.ShortDesc)
+        table = Gtk.Table()
         
         row = col = 0
         self.Checks = []
         sortedItems = sorted(self.Setting.Info[1][2].items(), key=EnumSettingKeyFunc)
         self.minVal = sortedItems[0][1]
         for key, value in sortedItems:
-            box = gtk.CheckButton(key)
+            box = Gtk.CheckButton(key)
             self.Checks.append((key, box))
             table.attach(box, col, col+1, row, row+1, TableDef, TableDef, TableX, TableX)
             box.connect('toggled', self.Changed)
@@ -808,15 +800,15 @@ class EnumFlagsSetting(Setting):
                 col = 0
                 row += 1
 
-        vbox = gtk.VBox()
-        vbox.pack_start(self.Reset, False, False)
+        vbox = Gtk.VBox()
+        vbox.pack_start(self.Reset, False, False, 0)
 
-        hbox = gtk.HBox()
-        hbox.pack_start(table, True, True)
-        hbox.pack_start(vbox, False, False)
+        hbox = Gtk.HBox()
+        hbox.pack_start(table, True, True, 0)
+        hbox.pack_start(vbox, False, False, 0)
 
         frame.add(hbox)
-        self.Box.pack_start(frame, True, True)
+        self.Box.pack_start(frame, True, True, 0)
 
     def _Read(self):
         for key, box in self.Checks:
@@ -840,8 +832,8 @@ class EnumFlagsSetting(Setting):
 class RestrictedStringFlagsSetting(Setting):
 
     def _Init(self):
-        frame = gtk.Frame(self.Setting.ShortDesc)
-        table = gtk.Table()
+        frame = Gtk.Frame(label=self.Setting.ShortDesc)
+        table = Gtk.Table()
         
         row = col = 0
         self.Checks = []
@@ -850,7 +842,7 @@ class RestrictedStringFlagsSetting(Setting):
         self.ItemsByValue = info[1]
         sortedItems = info[2]
         for key, value in sortedItems:
-            box = gtk.CheckButton(key)
+            box = Gtk.CheckButton(key)
             self.Checks.append((key, box))
             table.attach(box, col, col+1, row, row+1, TableDef, TableDef, TableX, TableX)
             box.connect('toggled', self.Changed)
@@ -859,15 +851,15 @@ class RestrictedStringFlagsSetting(Setting):
                 col = 0
                 row += 1
 
-        vbox = gtk.VBox()
-        vbox.pack_start(self.Reset, False, False)
+        vbox = Gtk.VBox()
+        vbox.pack_start(self.Reset, False, False, 0)
 
-        hbox = gtk.HBox()
-        hbox.pack_start(table, True, True)
-        hbox.pack_start(vbox, False, False)
+        hbox = Gtk.HBox()
+        hbox.pack_start(table, True, True, 0)
+        hbox.pack_start(vbox, False, False, 0)
 
         frame.add(hbox)
-        self.Box.pack_start(frame, True, True)
+        self.Box.pack_start(frame, True, True, 0)
 
     def _Read(self):
         for key, box in self.Checks:
@@ -893,48 +885,48 @@ class EditableActionSetting (StockSetting):
 
     def _Init (self, widget, action):
         StockSetting._Init(self)
-        alignment = gtk.Alignment (0, 0.5)
+        alignment = Gtk.Alignment (xalign=0, yalign=0.5)
         alignment.add (widget)
 
         self.Label.set_size_request(-1, -1)
 
-        editButton = gtk.Button ()
-        editButton.add (Image (name = gtk.STOCK_EDIT, type = ImageStock,
-                               size = gtk.ICON_SIZE_BUTTON))
+        editButton = Gtk.Button ()
+        editButton.add (Image (name = Gtk.STOCK_EDIT, type = ImageStock,
+                               size = Gtk.IconSize.BUTTON))
         editButton.set_tooltip_text(_("Edit %s" % self.Setting.ShortDesc))
         editButton.connect ("clicked", self.RunEditDialog)
 
         action = ActionImage (action)
-        self.Box.pack_start (action, False, False)
+        self.Box.pack_start (action, False, False, 0)
         self.Box.reorder_child (action, 0)
-        self.Box.pack_end (editButton, False, False)
-        self.Box.pack_end(alignment, False, False)
+        self.Box.pack_end (editButton, False, False, 0)
+        self.Box.pack_end(alignment, False, False, 0)
         self.Widget = widget
 
 
     def RunEditDialog (self, widget):
-        dlg = gtk.Dialog (_("Edit %s") % self.Setting.ShortDesc)
-        dlg.set_position (gtk.WIN_POS_CENTER_ON_PARENT)
+        dlg = Gtk.Dialog (_("Edit %s") % self.Setting.ShortDesc)
+        dlg.set_position (Gtk.WindowPosition.CENTER_ON_PARENT)
         dlg.set_transient_for (self.Widget.get_toplevel ())
-        dlg.add_button (gtk.STOCK_CANCEL, gtk.RESPONSE_CANCEL)
-        dlg.add_button (gtk.STOCK_OK, gtk.RESPONSE_OK).grab_default()
-        dlg.set_default_response (gtk.RESPONSE_OK)
+        dlg.add_button (Gtk.STOCK_CANCEL, Gtk.ResponseType.CANCEL)
+        dlg.add_button (Gtk.STOCK_OK, Gtk.ResponseType.OK).grab_default()
+        dlg.set_default_response (Gtk.ResponseType.OK)
         
-        entry = gtk.Entry (max = 200)
+        entry = Gtk.Entry (max_length = 200)
         entry.set_text (self.GetDialogText ())
-        entry.connect ("activate", lambda *a: dlg.response (gtk.RESPONSE_OK))
-        alignment = gtk.Alignment (0.5, 0.5, 1, 1)
+        entry.connect ("activate", lambda *a: dlg.response (Gtk.ResponseType.OK))
+        alignment = Gtk.Alignment (xalign=0.5, yalign=0.5, xscale=1, yscale=1)
         alignment.set_padding (10, 10, 10, 10)
         alignment.add (entry)
 
         entry.set_tooltip_text(self.Setting.LongDesc)
-        dlg.vbox.pack_start (alignment)
+        dlg.vbox.pack_start (alignment, True, True, 0)
         
         dlg.vbox.show_all ()
         ret = dlg.run ()
         dlg.destroy ()
 
-        if ret != gtk.RESPONSE_OK:
+        if ret != Gtk.ResponseType.OK:
             return
 
         self.HandleDialogText (entry.get_text ().strip ())
@@ -965,7 +957,7 @@ class KeySetting (EditableActionSetting):
             self.Read ()
 
     def ReorderKeyString (self, accel):
-        key, mods = gtk.accelerator_parse (accel)
+        key, mods = Gtk.accelerator_parse (accel)
         return GetAcceleratorName (key, mods)
 
     def GetDialogText (self):
@@ -1029,31 +1021,31 @@ class KeySetting (EditableActionSetting):
                 new = current.replace ("%s_R" % modifier, "")
             label.set_text (self.GetLabelText (new))
 
-        dlg = gtk.Dialog (_("Edit %s") % self.Setting.ShortDesc)
-        dlg.set_position (gtk.WIN_POS_CENTER_ALWAYS)
+        dlg = Gtk.Dialog (_("Edit %s") % self.Setting.ShortDesc)
+        dlg.set_position (Gtk.WindowPosition.CENTER_ALWAYS)
         dlg.set_transient_for (self.Widget.get_toplevel ())
         dlg.set_icon (self.Widget.get_toplevel ().get_icon ())
         dlg.set_modal (True)
-        dlg.add_button (gtk.STOCK_CANCEL, gtk.RESPONSE_CANCEL)
-        dlg.add_button (gtk.STOCK_OK, gtk.RESPONSE_OK).grab_default ()
-        dlg.set_default_response (gtk.RESPONSE_OK)
+        dlg.add_button (Gtk.STOCK_CANCEL, Gtk.ResponseType.CANCEL)
+        dlg.add_button (Gtk.STOCK_OK, Gtk.ResponseType.OK).grab_default ()
+        dlg.set_default_response (Gtk.ResponseType.OK)
 
-        mainBox = gtk.VBox ()
-        alignment = gtk.Alignment ()
+        mainBox = Gtk.VBox ()
+        alignment = Gtk.Alignment ()
         alignment.set_padding (10, 10, 10, 10)
         alignment.add (mainBox)
-        dlg.vbox.pack_start (alignment)
+        dlg.vbox.pack_start (alignment, True, True, 0)
 
-        checkButton = gtk.CheckButton (_("Enabled"))
+        checkButton = Gtk.CheckButton (_("Enabled"))
         active = len (self.current) \
                  and self.current.lower () not in ("disabled", "none")
         checkButton.set_active (active)
         checkButton.set_tooltip_text(self.Setting.LongDesc)
-        mainBox.pack_start (checkButton)
+        mainBox.pack_start (checkButton, True, True, 0)
 
-        box = gtk.VBox ()
+        box = Gtk.VBox ()
         checkButton.connect ("toggled", ShowHideBox, box, dlg)
-        mainBox.pack_start (box)
+        mainBox.pack_start (box, True, True, 0)
 
         currentMods = ""
         for mod in KeyModifier:
@@ -1062,22 +1054,22 @@ class KeySetting (EditableActionSetting):
         currentMods.rstrip ("|")
         modifierSelector = ModifierSelector (currentMods)
         modifierSelector.set_tooltip_text (self.Setting.LongDesc)
-        alignment = gtk.Alignment (0.5)
+        alignment = Gtk.Alignment (xalign=0.5)
         alignment.add (modifierSelector)
-        box.pack_start (alignment)
+        box.pack_start (alignment, True, True, 0)
 
-        key, mods = gtk.accelerator_parse (self.current)
+        key, mods = Gtk.accelerator_parse (self.current)
         grabber = KeyGrabber (key = key, mods = mods,
                               label = _("Grab key combination"))
         grabber.set_tooltip_text (self.Setting.LongDesc)
-        box.pack_start (grabber)
+        box.pack_start (grabber, True, True, 0)
 
-        label = gtk.Label (self.current)
+        label = Gtk.Label (self.current)
         label.set_tooltip_text (self.Setting.LongDesc)
-        alignment = gtk.Alignment (0.5, 0.5)
+        alignment = Gtk.Alignment (xalign=0.5, yalign=0.5)
         alignment.set_padding (15, 0, 0, 0)
         alignment.add (label)
-        box.pack_start (alignment)
+        box.pack_start (alignment, True, True, 0)
 
         modifierSelector.connect ("added", HandleModifierAdded, label)
         modifierSelector.connect ("removed", HandleModifierRemoved, label)
@@ -1091,7 +1083,7 @@ class KeySetting (EditableActionSetting):
         ret = dlg.run ()
         dlg.destroy ()
 
-        if ret != gtk.RESPONSE_OK:
+        if ret != Gtk.ResponseType.OK:
             return
 
         if not checkButton.get_active ():
@@ -1182,30 +1174,30 @@ class ButtonSetting (EditableActionSetting):
             else:
                 box.hide ()
                 dialog.resize (1, 1)
-        dlg = gtk.Dialog (_("Edit %s") % self.Setting.ShortDesc)
-        dlg.set_position (gtk.WIN_POS_CENTER_ALWAYS)
+        dlg = Gtk.Dialog (_("Edit %s") % self.Setting.ShortDesc)
+        dlg.set_position (Gtk.WindowPosition.CENTER_ALWAYS)
         dlg.set_transient_for (self.Widget.get_toplevel ())
         dlg.set_modal (True)
-        dlg.add_button (gtk.STOCK_CANCEL, gtk.RESPONSE_CANCEL)
-        dlg.add_button (gtk.STOCK_OK, gtk.RESPONSE_OK).grab_default ()
-        dlg.set_default_response (gtk.RESPONSE_OK)
+        dlg.add_button (Gtk.STOCK_CANCEL, Gtk.ResponseType.CANCEL)
+        dlg.add_button (Gtk.STOCK_OK, Gtk.ResponseType.OK).grab_default ()
+        dlg.set_default_response (Gtk.ResponseType.OK)
 
-        mainBox = gtk.VBox ()
-        alignment = gtk.Alignment ()
+        mainBox = Gtk.VBox ()
+        alignment = Gtk.Alignment ()
         alignment.set_padding (10, 10, 10, 10)
         alignment.add (mainBox)
-        dlg.vbox.pack_start (alignment)
+        dlg.vbox.pack_start (alignment, True, True, 0)
 
-        checkButton = gtk.CheckButton (_("Enabled"))
+        checkButton = Gtk.CheckButton (_("Enabled"))
         active = len (self.current) \
                  and self.current.lower () not in ("disabled", "none")
         checkButton.set_active (active)
         checkButton.set_tooltip_text (self.Setting.LongDesc)
-        mainBox.pack_start (checkButton)
+        mainBox.pack_start (checkButton, True, True, 0)
 
-        box = gtk.VBox ()
+        box = Gtk.VBox ()
         checkButton.connect ("toggled", ShowHideBox, box, dlg)
-        mainBox.pack_start (box)
+        mainBox.pack_start (box, True, True, 0)
 
         currentEdges = ""
         for edge in Edges:
@@ -1214,7 +1206,7 @@ class ButtonSetting (EditableActionSetting):
         currentEdges.rstrip ("|")
         edgeSelector = SingleEdgeSelector (currentEdges)
         edgeSelector.set_tooltip_text(self.Setting.LongDesc)
-        box.pack_start (edgeSelector)
+        box.pack_start (edgeSelector, True, True, 0)
 
         currentMods = ""
         for mod in KeyModifier:
@@ -1223,9 +1215,9 @@ class ButtonSetting (EditableActionSetting):
         currentMods.rstrip ("|")
         modifierSelector = ModifierSelector (currentMods)
         modifierSelector.set_tooltip_text(self.Setting.LongDesc)
-        box.pack_start (modifierSelector)
+        box.pack_start (modifierSelector, True, True, 0)
 
-        buttonCombo = gtk.combo_box_new_text ()
+        buttonCombo = Gtk.ComboBoxText ()
         currentButton = 1
         for i in range (99, 0, -1):
             if "Button%d" % i in self.current:
@@ -1241,14 +1233,15 @@ class ButtonSetting (EditableActionSetting):
         else:
             buttonCombo.set_active (currentButton - 1)
         buttonCombo.set_tooltip_text(self.Setting.LongDesc)
-        box.pack_start (buttonCombo)
+        box.pack_start (buttonCombo, True, True, 0)
 
         dlg.vbox.show_all ()
         ShowHideBox (checkButton, box, dlg)
         ret = dlg.run ()
+        button = buttonCombo.get_active_text ()
         dlg.destroy ()
 
-        if ret != gtk.RESPONSE_OK:
+        if ret != Gtk.ResponseType.OK:
             return
 
         if not checkButton.get_active ():
@@ -1257,7 +1250,6 @@ class ButtonSetting (EditableActionSetting):
 
         edges = edgeSelector.current
         modifiers = modifierSelector.current
-        button = buttonCombo.get_active_text ()
 
         edges = edges.split ("|")
         if len (edges):
@@ -1277,12 +1269,22 @@ class ButtonSetting (EditableActionSetting):
     def ButtonEdited (self, button):
         '''Button edited callback'''
         if button == "Button1":
-            warning = WarningDialog (self.Widget.get_toplevel (),
-                                     _("Using Button1 without modifiers can \
+
+            warning = Gtk.MessageDialog (transient_for = self.Widget.get_toplevel (),
+                                         destroy_with_parent = True,
+                                         message_type = Gtk.MessageType.WARNING,
+                                         buttons = Gtk.ButtonsType.YES_NO,
+                                         title = _("Warning"))
+
+            warning.set_markup (GLib.markup_escape_text (_("Using Button1 without modifiers can \
 prevent any left click and thus break your configuration. Do you really want \
-to set \"%s\" button to Button1 ?") % self.Setting.ShortDesc)
+to set \"%s\" button to Button1 ?") % self.Setting.ShortDesc))
+
+            warning.set_position (Gtk.WindowPosition.CENTER)
+            warning.connect_after ("response", lambda *args: warning.destroy ())
+
             response = warning.run ()
-            if response != gtk.RESPONSE_YES:
+            if response != Gtk.ResponseType.YES:
                 return
         conflict = ButtonConflict (self.Setting, button)
         if conflict.Resolve (GlobalUpdater):
@@ -1345,27 +1347,27 @@ class EdgeSetting (EditableActionSetting):
         self.Button.set_label (label)
 
     def RunEdgeSelector (self, widget):
-        dlg = gtk.Dialog (_("Edit %s") % self.Setting.ShortDesc)
-        dlg.set_position (gtk.WIN_POS_CENTER_ON_PARENT)
+        dlg = Gtk.Dialog (_("Edit %s") % self.Setting.ShortDesc)
+        dlg.set_position (Gtk.WindowPosition.CENTER_ON_PARENT)
         dlg.set_transient_for (self.Widget.get_toplevel ())
         dlg.set_modal (True)
-        dlg.add_button (gtk.STOCK_CANCEL, gtk.RESPONSE_CANCEL)
-        dlg.add_button (gtk.STOCK_OK, gtk.RESPONSE_OK).grab_default()
-        dlg.set_default_response (gtk.RESPONSE_OK)
+        dlg.add_button (Gtk.STOCK_CANCEL, Gtk.ResponseType.CANCEL)
+        dlg.add_button (Gtk.STOCK_OK, Gtk.ResponseType.OK).grab_default()
+        dlg.set_default_response (Gtk.ResponseType.OK)
         
         selector = SingleEdgeSelector (self.current)
-        alignment = gtk.Alignment ()
+        alignment = Gtk.Alignment ()
         alignment.set_padding (10, 10, 10, 10)
         alignment.add (selector)
 
         selector.set_tooltip_text (self.Setting.LongDesc)
-        dlg.vbox.pack_start (alignment)
+        dlg.vbox.pack_start (alignment, True, True, 0)
         
         dlg.vbox.show_all ()
         ret = dlg.run ()
         dlg.destroy ()
 
-        if ret != gtk.RESPONSE_OK:
+        if ret != Gtk.ResponseType.OK:
             return
 
         self.EdgeEdited (selector.current)
@@ -1391,7 +1393,7 @@ class BellSetting (BoolSetting):
     def _Init (self):
         BoolSetting._Init (self)
         bell = ActionImage ("bell")
-        self.Box.pack_start (bell, False, False)
+        self.Box.pack_start (bell, False, False, 0)
         self.Box.reorder_child (bell, 0)
 
 def MakeStringSetting (setting, List=False):
@@ -1471,13 +1473,13 @@ class SubGroupArea(object):
         self.Name = name
         settings = sorted(GetSettings(subGroup), key=SettingKeyFunc)
         if not name:
-            self.Child = self.Widget = gtk.VBox()
+            self.Child = self.Widget = Gtk.VBox()
         else:
-            self.Widget = gtk.Frame()
-            self.Expander = gtk.Expander(name)
+            self.Widget = Gtk.Frame()
+            self.Expander = Gtk.Expander(label=name)
             self.Widget.add(self.Expander)
             self.Expander.set_expanded(False)
-            self.Child = gtk.VBox()
+            self.Child = Gtk.VBox()
             self.Expander.add(self.Child)
 
         self.Child.set_spacing(TableX)
@@ -1487,7 +1489,7 @@ class SubGroupArea(object):
         if len(settings) > 1 and HasOnlyType(settings, 'List'):
             multiList = MultiListSetting(Settings=settings)
             multiList.Read()
-            self.Child.pack_start(multiList.EBox, True, True)
+            self.Child.pack_start(multiList.EBox, True, True, 0)
             self.MySettings.append(multiList)
             self.Empty = False
             if name:
@@ -1501,7 +1503,7 @@ class SubGroupArea(object):
                 setting = MakeSetting(setting)
                 if setting is not None:
                     setting.Read()
-                    self.Child.pack_start(setting.EBox, True, True)
+                    self.Child.pack_start(setting.EBox, True, True, 0)
                     self.MySettings.append(setting)
                     self.Empty = False
 
