@@ -92,7 +92,8 @@ ExpoScreen::doExpo (CompAction         *action,
 		    CompAction::State  state,
 		    CompOption::Vector &options)
 {
-    if (screen->otherGrabExist ("expo", NULL))
+    if (screen->otherGrabExist ("expo", NULL) || 
+	(screen->vpSize ().width () == 1 && screen->vpSize ().height () == 1))
 	return false;
 
     if (!expoMode)
@@ -186,6 +187,10 @@ ExpoScreen::nextVp (CompAction         *action,
     if (!expoMode)
 	return false;
 
+    CompPoint pointer (pointerX, pointerY);
+    if (!screen->workArea ().contains (pointer))
+	return false;
+
     unsigned int newX = selectedVp.x () + 1;
     unsigned int newY = selectedVp.y ();
 
@@ -213,6 +218,10 @@ ExpoScreen::prevVp (CompAction         *action,
     if (!expoMode)
 	return false;
 
+    CompPoint pointer (pointerX, pointerY);
+    if (!screen->workArea ().contains (pointer))
+	return false;
+
     int newX = selectedVp.x () - 1;
     int newY = selectedVp.y ();
 
@@ -402,10 +411,18 @@ ExpoScreen::preparePaint (int msSinceLastPaint)
     float val = (static_cast <float> (msSinceLastPaint) / 1000.0f) /
 		optionGetZoomTime ();
 
-    if (expoMode)
-	expoCam = MIN (1.0, expoCam + val);
-    else
-	expoCam = MAX (0.0, expoCam - val);
+    if (optionGetExpoAnimation() != ExpoScreen::ExpoAnimationNone) {
+	if (expoMode)
+	    expoCam = MIN (1.0, expoCam + val);
+	else
+	    expoCam = MAX (0.0, expoCam - val);
+    }
+    else {
+	if (expoMode)
+	    expoCam = 1;
+	else
+	    expoCam = 0;
+    }
 
     if (expoCam)
     {
@@ -921,6 +938,8 @@ ExpoScreen::paintWall (const GLScreenPaintAttrib &attrib,
     /* translate expo to center */
     sTransform.translate (vpSize.x () * sx * -0.5 + offsetInWorldCoordX,
 			  vpSize.y () * sy *  0.5 - offsetInWorldCoordY, 0.0f);
+
+    worldScaleFactorY -= (float)optionGetYBottomOffset () / (float)output->height ();
     sTransform.scale (worldScaleFactorX, worldScaleFactorY, 1.0f);
 
 
@@ -1503,7 +1522,7 @@ ExpoWindow::glPaint (const GLWindowPaintAttrib &attrib,
 	    float yS = 1.0 + ((o->height () / (float) window->height ()) - 1.0f) * sigmoidProgress (eScreen->expoCam);
 	    float xS = 1.0 + ((o->width () / (float) window->width ()) - 1.0f) * sigmoidProgress (eScreen->expoCam);
 	    wTransform.translate (window->x () + window->width (),
-				  window->y () + window->height (),
+				  window->y () + window->height () + eScreen->optionGetYBottomOffset (),
 				  0.0f);
 	    wTransform.scale (xS, yS, 1.0f);
 	    wTransform.translate (-(window->x () + window->width ()),
