@@ -1207,14 +1207,14 @@ TEST_F(CCSGSettingsTestIndependent, TestGetNameForCCSSetting)
 
 TEST_F(CCSGSettingsTestIndependent, TestReadVariantIsValidNULL)
 {
-    EXPECT_FALSE (checkReadVariantIsValid (NULL, TypeNum, "foo/bar"));
+    EXPECT_FALSE (checkReadVariantIsValid (NULL, TypeNum, "foo/bar", "key"));
 }
 
 TEST_F(CCSGSettingsTestIndependent, TestReadVariantIsValidTypeBad)
 {
     GVariant *v = g_variant_new ("i", 1);
 
-    EXPECT_FALSE (checkReadVariantIsValid (v, TypeString, "foo/bar"));
+    EXPECT_FALSE (checkReadVariantIsValid (v, TypeString, "foo/bar", "key"));
 
     g_variant_unref (v);
 }
@@ -1223,7 +1223,7 @@ TEST_F(CCSGSettingsTestIndependent, TestReadVariantIsValidTypeGood)
 {
     GVariant *v = g_variant_new ("i", 1);
 
-    EXPECT_TRUE (checkReadVariantIsValid (v, TypeInt, "foo/bar"));
+    EXPECT_TRUE (checkReadVariantIsValid (v, TypeInt, "foo/bar", "key"));
 
     g_variant_unref (v);
 }
