@@ -1298,16 +1298,7 @@ GLWindow::glDrawTexture (GLTexture          *texture,
 	priv->vertexBuffer->render (transform, attrib);
     #endif
 
-    for (std::list<const GLShaderData*>::const_iterator it = priv->shaders.begin();
-         it != priv->shaders.end();
-         ++it)
-    {
-	if ((*it)->isCached != true)
-	{
-	    delete *it;
-	}
-    }
-    priv->shaders.clear ();
+    clearShaders ();
     texture->disable ();
 
     if (mask & PAINT_WINDOW_BLEND_MASK)
@@ -1324,7 +1315,7 @@ GLWindow::glDraw (const GLMatrix     &transform,
 			      attrib, region, mask)
 
     const CompRegion &reg = (mask & PAINT_WINDOW_TRANSFORMED_MASK) ?
-                            CompRegion::infinite () : region;
+                            infiniteRegion : region;
 
     if (reg.isEmpty ())
 	return true;
@@ -1367,6 +1358,14 @@ GLWindow::glDraw (const GLMatrix     &transform,
     return true;
 }
 
+void
+GLWindow::glTransformationComplete (const GLMatrix   &matrix,
+				    const CompRegion &reg,
+				    unsigned int     mask)
+{
+    WRAPABLE_HND_FUNCTN (glTransformationComplete, matrix, reg, mask);
+}
+
 bool
 GLWindow::glPaint (const GLWindowPaintAttrib &attrib,
 		   const GLMatrix            &transform,
@@ -1384,6 +1383,8 @@ GLWindow::glPaint (const GLWindowPaintAttrib &attrib,
 
     priv->lastMask = mask;
 
+    glTransformationComplete (transform, region, mask);
+
     if (mask & PAINT_WINDOW_OCCLUSION_DETECTION_MASK)
     {
 	if (mask & PAINT_WINDOW_TRANSFORMED_MASK)
