@@ -1821,6 +1821,11 @@ CompScreenImpl::_handleEvent (XEvent *event)
 	    if (w)
 		w->priv->updateStartupId ();
 	}
+	else if (event->xproperty.atom == Atoms::gnomeRepresentativeColors)
+	{
+	    if (event->xproperty.window == privateScreen.rootWindow())
+		privateScreen.updateAverageColor (event->xproperty.atom);
+	}
 	else if (event->xproperty.atom == XA_WM_CLASS)
 	{
 	    w = findWindow (event->xproperty.window);
@@ -2271,14 +2276,14 @@ static const unsigned short _NET_WM_STATE_TOGGLE = 2;
 				}
 			    }
 
-			    active->changeState (active->focused () ?
+			    active->changeState (active->focused () || (w->type () & APPEARS_FOCUSED_MASK) ?
 						 active->state () | CompWindowStateFocusedMask :
 						 active->state () & ~CompWindowStateFocusedMask);
 
 			    active->priv->updatePassiveButtonGrabs ();
 			}
 
-			if (w->focused ())
+			if (w->focused () || (w->type () & APPEARS_FOCUSED_MASK))
 			    state |= w->state () | CompWindowStateFocusedMask;
 			else
 			    state &= w->state () & ~CompWindowStateFocusedMask;
