@@ -965,7 +965,7 @@ StaticSwitchScreen::glPaintOutput (const GLScreenPaintAttrib &sAttrib,
 		    sw->cWindow->damaged ())
 		{
 		    sw->gWindow->glPaint (sw->gWindow->paintAttrib (),
-					  sTransform, infiniteRegion, 0);
+					  sTransform, CompRegion::infinite (), 0);
 		}
 	    }
 	}
